/// Modem config element
typedef struct dev_conf {
	int vid;		///< vendor id
	int pid;		///< product id
	char type;		///< modem type - 'C' for CDMA, 'W' for GPRS/UMTS/LTE, 'T' (unused) for TD-SCDMA
	int data_port;	///< main port for data communication number
	int data_cls;	///< interface class (used if device class == 0)
	int data_sub;	///< interface subclass
	int data_prot;	///< protocol type
	int ui_port;	///< user interface port for modem controlling, status sms, ussd and so on
	int ui_cls;		///< interface class (used if device class == 0)
	int ui_sub;		///< interface subclass
	int ui_prot;	///< protocol type
} dev_conf_t;

#define MODEM_DEVICE_AND_INTERFACE_NUMBER(vend,prod,typ,dpt,upt) \
	.vid = (vend), .pid = (prod), .type = (typ), \
	.data_port = (dpt), .ui_port = (upt), \
	.data_cls = -1, .data_sub = -1, .data_prot = -1, \
	.ui_cls = -1, .ui_sub = -1, .ui_prot = -1

#define MODEM_VENDOR_AND_INTERFACE_INFO(vend,typ,dcl,dsc,dpr,ucl,usc,upr) \
	.vid = (vend), .pid = -1, .type = (typ), \
	.data_port = -1, .ui_port = -1, \
	.data_cls = (dcl), .data_sub = (dsc), .data_prot = (dpr), \
	.ui_cls = (ucl), .ui_sub = (usc), .ui_prot = (upr)

#define HUAWEI_VENDOR_ID			0x12d1
#define ZTE_VENDOR_ID				0x19d2

static const struct dev_conf modem_ids[] = {
	// AnyDATA Corporation
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x16d5, 0x6501, TYPE_CDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x16d5, 0x6502, TYPE_CDMA, 0, 2) },
	// Axesstel
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x1726, 0x1000, TYPE_CDMA, 0, 1) },
	// CMOTECH CDMA Technologies
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x16d8, 0x6803, TYPE_CDMA, 0, 1) },
	// Huawei Technologies Co.,Ltd
	// 22.001.18.22.209
	{ MODEM_VENDOR_AND_INTERFACE_INFO(HUAWEI_VENDOR_ID, TYPE_WCDMA, 0xff, 0x01, 0x01, 0xff, 0x01, 0x02) },
	{ MODEM_VENDOR_AND_INTERFACE_INFO(HUAWEI_VENDOR_ID, TYPE_WCDMA, 0xff, 0x01, 0x31, 0xff, 0x01, 0x32) },
	{ MODEM_VENDOR_AND_INTERFACE_INFO(HUAWEI_VENDOR_ID, TYPE_WCDMA, 0xff, 0x02, 0x01, 0xff, 0x02, 0x02) },
	{ MODEM_VENDOR_AND_INTERFACE_INFO(HUAWEI_VENDOR_ID, TYPE_WCDMA, 0xff, 0x02, 0x31, 0xff, 0x02, 0x32) },
	{ MODEM_VENDOR_AND_INTERFACE_INFO(HUAWEI_VENDOR_ID, TYPE_WCDMA, 0xff, 0x01, 0x10, 0xff, 0x01, 0x12) },
	{ MODEM_VENDOR_AND_INTERFACE_INFO(HUAWEI_VENDOR_ID, TYPE_WCDMA, 0xff, 0x01, 0x61, 0xff, 0x01, 0x62) },
	{ MODEM_VENDOR_AND_INTERFACE_INFO(HUAWEI_VENDOR_ID, TYPE_WCDMA, 0xff, 0x02, 0x10, 0xff, 0x02, 0x12) },
	{ MODEM_VENDOR_AND_INTERFACE_INFO(HUAWEI_VENDOR_ID, TYPE_WCDMA, 0xff, 0x02, 0x61, 0xff, 0x02, 0x62) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1001, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a8, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1003, TYPE_WCDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a9, TYPE_WCDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c1d, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1004, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1401, TYPE_WCDMA, 3, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1402, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1403, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1405, TYPE_WCDMA, 3, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1406, TYPE_WCDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1408, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1409, TYPE_WCDMA, 3, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x140a, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x140b, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x140c, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14aa, TYPE_WCDMA, 0, 3) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x140d, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x140e, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x140f, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1411, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1412, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1413, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1414, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1415, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1416, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1417, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1418, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1419, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x141a, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x141b, TYPE_WCDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x141c, TYPE_WCDMA, 1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x141d, TYPE_WCDMA, 2, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x141e, TYPE_WCDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x141f, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1420, TYPE_WCDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1421, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1422, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1422, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1427, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1428, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1429, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x142a, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1448, TYPE_WCDMA, 0, 1) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x142d, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x142e, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x142f, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1430, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1431, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1432, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1433, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1434, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1435, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1436, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1437, TYPE_WCDMA, -1, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1438, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1439, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x143a, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x143b, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x143e, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14ac, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14ab, TYPE_WCDMA, 0, 4) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x144a, TYPE_WCDMA, 0, 2) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1450, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1451, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1452, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1453, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1454, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1455, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1456, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1457, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1458, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1459, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x145a, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x145b, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x145c, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x145d, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x145e, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x145f, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1460, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1461, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1462, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1463, TYPE_WCDMA, 0, 2) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1404, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1407, TYPE_WCDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x142b, TYPE_WCDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x144b, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1464, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1465, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1466, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1467, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1468, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1469, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x146a, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x146b, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x146c, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x146d, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x146e, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x146f, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1470, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1471, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1472, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1473, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1474, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1475, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1476, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1477, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1478, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1479, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x147a, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x147b, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x147c, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x147d, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x147e, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x147f, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1480, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1481, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14c8, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14c9, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14ca, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14cb, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14cc, TYPE_WCDMA, 0, 4) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1491, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1492, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1493, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1494, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1495, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1496, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1497, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1498, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1499, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x149a, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x149b, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x149c, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x149d, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x149e, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x149f, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a0, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a1, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a2, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a3, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a4, TYPE_WCDMA, -1, 0) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1482, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1483, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1484, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1485, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1486, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1487, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1488, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1489, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x148a, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x148b, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1510, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1511, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1512, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1513, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1514, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1516, TYPE_WCDMA, 0, 4) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1d03, TYPE_TDSCDMA, 1, 5) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1d09, TYPE_TDSCDMA, 1, 5) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1d50, TYPE_TDSCDMA, 1, 3) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c05, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c06, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c07, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c08, TYPE_WCDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c09, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c0a, TYPE_WCDMA, -1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c0c, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c0d, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c0e, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c0f, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c10, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c11, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c12, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c13, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c14, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c15, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c16, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c23, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1444, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1445, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x144e, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x144f, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14ae, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14af, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14b0, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14b1, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a5, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a6, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14a7, TYPE_WCDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c17, TYPE_WCDMA, -1, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c18, TYPE_WCDMA, -1, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c19, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x1c1a, TYPE_WCDMA, 0, 3) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14b8, TYPE_WCDMA, -1, 1) },

	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14d2, TYPE_WCDMA, 0, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14d3, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(HUAWEI_VENDOR_ID, 0x14d4, TYPE_WCDMA, -1, 1) },
	// ZTE,Incorporated
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0001, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0002, TYPE_WCDMA, 2, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0003, TYPE_CDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0004, TYPE_CDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0005, TYPE_CDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0006, TYPE_CDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0007, TYPE_CDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0008, TYPE_CDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0009, TYPE_CDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x000a, TYPE_CDMA, 0, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0012, TYPE_WCDMA, 2, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0015, TYPE_WCDMA, 1, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0016, TYPE_WCDMA, 2, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0017, TYPE_WCDMA, 2, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0018, TYPE_WCDMA, 2, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0019, TYPE_WCDMA, 2, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0021, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0024, TYPE_WCDMA, 2, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0025, TYPE_WCDMA, 2, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0030, TYPE_WCDMA, 1, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0031, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0033, TYPE_WCDMA, 4, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0037, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0039, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0042, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0043, TYPE_WCDMA, 3, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0048, TYPE_WCDMA, 4, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0049, TYPE_WCDMA, 4, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0052, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0054, TYPE_WCDMA, 1, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0055, TYPE_WCDMA, 2, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0057, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0058, TYPE_WCDMA, 3, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0061, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0063, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0064, TYPE_WCDMA, 2, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0066, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0078, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0082, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0091, TYPE_WCDMA, 4, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0104, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0106, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0108, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0113, TYPE_WCDMA, 4, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0117, TYPE_WCDMA, 2, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0118, TYPE_WCDMA, 4, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0121, TYPE_WCDMA, 4, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0122, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0123, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0124, TYPE_WCDMA, 4, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0125, TYPE_WCDMA, 5, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0126, TYPE_WCDMA, 4, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0128, TYPE_WCDMA, 4, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x0167, TYPE_WCDMA, 3, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x1007, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x1008, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x1010, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x1218, TYPE_WCDMA, 2, 4) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x1515, TYPE_WCDMA, 0, 5) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x1544, TYPE_WCDMA, 1, 0) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x2002, TYPE_WCDMA, 3, 1) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(ZTE_VENDOR_ID, 0x2003, TYPE_WCDMA, 3, 1) },
	// longcheer Alcatel One Touch X020
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x1c9e, 0x6060, TYPE_WCDMA, 0, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x1c9e, 0x6061, TYPE_WCDMA, 0, 2) },
	// tamobile Alcatel One Touch X060s
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x1bbb, 0x0000, TYPE_WCDMA, 3, 1) },
	// WeTelecom
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x22de, 0x6801, TYPE_CDMA, 0, 2) },
	// Qisda
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x1da5, 0x4512, TYPE_WCDMA, 2, 3) },
	// Airplus MCD-650
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x1edf, 0x6004, TYPE_CDMA, 0, 3) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x1011, 0x3198, TYPE_CDMA, 0, 1) },
	// Novatel Wireless Inc
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x1410, 0x2110, TYPE_CDMA, 0, 1) },
	// Pantech UM175
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x106c, 0x3715, TYPE_CDMA, 0, 2) },
	// Pantech UM185
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x106c, 0x3717, TYPE_CDMA, 0, 2) },
	// Quanta
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x0408, 0xea26, TYPE_WCDMA, 2, 1) },
	// Yota Devices
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x0408, 0xea40, TYPE_WCDMA, 3, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x0408, 0xea41, TYPE_WCDMA, 3, 2) },
	{ MODEM_DEVICE_AND_INTERFACE_NUMBER(0x0408, 0xea42, TYPE_WCDMA, 3, 2) },
	{ }
};

